#ifndef MZ_h
#define MZ_h

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--


#include <algorithm>
#include <iomanip>
#include <iostream>
#include <map>
#include <string>

#include <bpipe/datatypes.h>
#include <funtools.h>
#include <paramxx/ParFile.h>
#include <suplib/debug.h>
#include <suplib/lists.h>
#include <suplib/str.h>
#include <tracefctxx/TraceFct.h>
#include <Exception/Exception.h>
#include <rdbxx/RDB.h>

#include "Random.h"
#include "cmp.h"

using namespace std;


class MZ {
 public:
  MZ( const string& id="zone_id" ) throw ( Exception );
  virtual ~MZ( void );

  enum ScatterStatus {
    OK 			   = 0x0000,
    SCATTERED_INTO_SURFACE = 0x0001,
    BACKSCATTERED 	   = 0x0002,
    IP_EXTRAPOLATED 	   = 0x0004,
    OP_EXTRAPOLATED 	   = 0x0008
  };

  int scatter( ParFile& pf, const long id, const double energy,
	       const DVector3& surface_norm, DVector3& direction,
	       double& theta_o, double& phi_o
	       );

  virtual bool lookUpScatterAngle( double energy, double sin_alpha, double rannum,
				   bool inplane, double& theta ) = 0;
  virtual double lookUpTheta( const int idx, double rannum ) = 0;
  double interpolate( double x, double xa, double ya, double xb, double yb );
  
  const string& id( void ) const { return _id; };
  const int nRows( void ) const { return _nrows; };
  const char *data( void ) const { return _data; };

  const double zmin( void ) const { return _zmin; };
  const double zmax( void ) const { return _zmax; };

  void zmin( const double d ) { _zmin = d; };
  void zmax( const double d ) { _zmax = d; };


 protected:
  virtual void init( const string& hduname ) throw ( Exception ) = 0;
  virtual void verifyCards( Fun fun ) throw ( Exception );
  virtual void verifyColumns( Fun fun, vector<string>& names ) throw ( Exception );
  virtual void verifyData( Fun fun ) throw( Exception );

  string _id;
  double _zmin;
  double _zmax;
  
  char *_data;
  int _rowsize;
  int _nrows;
  map<string,int> _offsets;
  map<string,int> _ns;

};

class mz_less : public less<MZ*> {
 public:
  bool operator() ( MZ* const& t1, MZ* const& t2 ) {
    return t1->zmin() < t2->zmax();
  };
};
#endif // #ifndef MZ_h
