// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#ifndef MZO_h
#define MZO_h

#include <string>

#include <funtools.h>
#include <tracefctxx/TraceFct.h>
#include <Exception/Exception.h>

#include "MZ.h"

using namespace std;

class MZO : public MZ {
 public:
  MZO( const string& id, const string& hduname, RDB& rdb ) throw ( Exception );
  ~MZO( void ) { };

  virtual bool lookUpScatterAngle( double energy, double sin_alpha, double rannum, bool inplane, double& theta );
  virtual double lookUpTheta( const int idx, double rannum );

  double esa( const int idx ) { return *(double*)(_data + _offsets["esa"] + _rowsize * idx); };
  double *pdist( const int idx ) { return (double*)(_data + _offsets["pdist"] + _rowsize * idx); };
  double p0( const int idx ) { return *(double*)(_data + _offsets["p0"] + _rowsize * idx); };
  double ptop( const int idx ) { return *(double*)(_data + _offsets["ptop"] + _rowsize * idx); };
  double pdelta( const int idx ) { return *(double*)(_data + _offsets["pdelta"] + _rowsize * idx); };
  double powtop( const int idx ) { return *(double*)(_data + _offsets["powtop"] + _rowsize * idx); };
  double normtop( const int idx ) { return *(double*)(_data + _offsets["normtop"] + _rowsize * idx); };

 protected:
  virtual void init( const string& hduname ) throw ( Exception );

  static const string _esa;
  static const string _pdist;
  static const string _p0;
  static const string _ptop;
  static const string _pdelta;
  static const string _powtop;
  static const string _normtop;
  
};

#endif // #ifndef MZO_h
